@@ -82,6 +82,7 @@ DIFFERENT_DEPENDENCIES = [
         (None, "({ws}==={ws}arbitrarystring{ws})"),
         (None, "=={ws}1.0"),
         (None, "({ws}=={ws}1.0{ws})"),
+        (None, "=={ws}1.0-alpha"),
         (None, "<={ws}1!3.0.0.rc2"),
         (None, ">{ws}2.2{ws},{ws}<{ws}3"),
         (None, "(>{ws}2.2{ws},{ws}<{ws}3)"),
@@ -278,11 +279,48 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_PARENTHESIS\n"
+            "Expected matching RIGHT_PARENTHESIS for LEFT_PARENTHESIS, "
+            "after version specifier\n"
             "    name (>= 1.0\n"
             "         ~~~~~~~^"
         )
 
+    def test_error_when_prefix_match_is_used_incorrectly(self) -> None:
+        # GIVEN
+        to_parse = "black (>=20.*) ; extra == 'format'"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            ".* suffix can only be used with `==` or `!=` operators\n"
+            "    black (>=20.*) ; extra == 'format'\n"
+            "           ~~~~~^"
+        )
+
+    @pytest.mark.parametrize("operator", [">=", "<=", ">", "<", "~="])
+    def test_error_when_local_version_label_is_used_incorrectly(
+        self, operator: str
+    ) -> None:
+        # GIVEN
+        to_parse = f"name {operator} 1.0+local.version.label"
+        op_tilde = len(operator) * "~"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            "Local version label can only be used with `==` or `!=` operators\n"
+            f"    name {operator} 1.0+local.version.label\n"
+            f"         {op_tilde}~~~~^"
+        )
+
     def test_error_when_bracket_not_closed_correctly(self) -> None:
         # GIVEN
         to_parse = "name[bar, baz >= 1.0"
@@ -294,7 +332,8 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_BRACKET\n"
+            "Expected matching RIGHT_BRACKET for LEFT_BRACKET, "
+            "after extras\n"
             "    name[bar, baz >= 1.0\n"
             "        ~~~~~~~~~~^"
         )
@@ -310,7 +349,8 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected closing RIGHT_BRACKET\n"
+            "Expected matching RIGHT_BRACKET for LEFT_BRACKET, "
+            "after extras\n"
             "    name[bar, baz\n"
             "        ~~~~~~~~~^"
         )
@@ -331,6 +371,23 @@ class TestRequirementParsing:
             "           ~~~~~~~~~~~~~~~~~~~~~~^"
         )
 
+    def test_error_marker_bracket_unclosed(self) -> None:
+        # GIVEN
+        to_parse = "name; (extra == 'example'"
+
+        # WHEN
+        with pytest.raises(InvalidRequirement) as ctx:
+            Requirement(to_parse)
+
+        # THEN
+        assert ctx.exconly() == (
+            "packaging.requirements.InvalidRequirement: "
+            "Expected matching RIGHT_PARENTHESIS for LEFT_PARENTHESIS, "
+            "after marker expression\n"
+            "    name; (extra == 'example'\n"
+            "          ~~~~~~~~~~~~~~~~~~~^"
+        )
+
     def test_error_no_url_after_at(self) -> None:
         # GIVEN
         to_parse = "name @ "
@@ -492,9 +549,9 @@ class TestRequirementParsing:
         # THEN
         assert ctx.exconly() == (
             "packaging.requirements.InvalidRequirement: "
-            "Expected version after operator\n"
+            "Expected end or semicolon (after name and no valid version specifier)\n"
             "    name==\n"
-            "          ^"
+            "        ^"
         )
 
     def test_error_on_missing_op_after_name(self) -> None:
@@ -580,14 +637,14 @@ class TestRequirementBehaviour:
         self, extras: str, url_or_specifier: str, marker: str
     ) -> None:
         # GIVEN
-        to_parse = f"name{extras}{url_or_specifier}{marker}".strip()
+        to_parse = f"name{extras}{url_or_specifier}{marker}"
 
         # WHEN
         req = Requirement(to_parse)
 
         # THEN
-        assert str(req) == to_parse
-        assert repr(req) == f"<Requirement({to_parse!r})>"
+        assert str(req) == to_parse.strip()
+        assert repr(req) == f"<Requirement({to_parse.strip()!r})>"
 
     @pytest.mark.parametrize("dep1, dep2", EQUAL_DEPENDENCIES)
     def test_equal_reqs_equal_hashes(self, dep1: str, dep2: str) -> None:
