@@ -234,7 +234,6 @@ class TestMacOSPlatforms:
 
         platforms = list(tags.mac_platforms(arch="x86_64"))
         if (major, minor) == ("10", "16"):
-            print(platforms, "macosx_11+")
             # For 10.16, the real version is at least 11.0.
             prefix, major, minor, _ = platforms[0].split("_", maxsplit=3)
             assert prefix == "macosx"
@@ -242,7 +241,6 @@ class TestMacOSPlatforms:
             assert minor == "0"
         else:
             expected = f"macosx_{major}_{minor}_"
-            print(platforms, expected)
             assert platforms[0].startswith(expected)
 
     def test_version_detection_10_15(self, monkeypatch):
@@ -252,7 +250,6 @@ class TestMacOSPlatforms:
         expected = "macosx_10_15_"
 
         platforms = list(tags.mac_platforms(arch="x86_64"))
-        print(platforms, expected)
         assert platforms[0].startswith(expected)
 
     def test_version_detection_compatibility(self, monkeypatch):
@@ -270,7 +267,6 @@ class TestMacOSPlatforms:
         unexpected = "macosx_10_16_"
 
         platforms = list(tags.mac_platforms(arch="x86_64"))
-        print(platforms, unexpected)
         assert not platforms[0].startswith(unexpected)
 
     @pytest.mark.parametrize("arch", ["x86_64", "i386"])
@@ -603,6 +599,13 @@ def test_platform_tags(platform_name, dispatch_func, monkeypatch):
     assert tags.platform_tags() == expected
 
 
+def test_platform_tags_space(monkeypatch):
+    """Ensure spaces in platform tags are normalized to underscores."""
+    monkeypatch.setattr(platform, "system", lambda: "Isilon OneFS")
+    monkeypatch.setattr(sysconfig, "get_platform", lambda: "isilon onefs")
+    assert list(tags.platform_tags()) == ["isilon_onefs"]
+
+
 class TestCPythonABI:
     @pytest.mark.parametrize(
         "py_debug,gettotalrefcount,result",
@@ -774,6 +777,12 @@ class TestCPythonTags:
         result = list(tags.cpython_tags((3, 11), abis=["whatever"]))
         assert tags.Tag("cp311", "whatever", "plat1") in result
 
+    def test_platform_name_space_normalization(self, monkeypatch):
+        """Ensure that spaces are translated to underscores in platform names."""
+        monkeypatch.setattr(sysconfig, "get_platform", lambda: "isilon onefs")
+        for tag in tags.cpython_tags():
+            assert " " not in tag.platform
+
     def test_major_only_python_version(self):
         result = list(tags.cpython_tags((3,), ["abi"], ["plat"]))
         assert result == [
@@ -823,28 +832,90 @@ class TestCPythonTags:
 
 
 class TestGenericTags:
-    @pytest.mark.skipif(
-        not sysconfig.get_config_var("SOABI"), reason="SOABI not defined"
-    )
-    def test__generic_abi_soabi_provided(self):
-        abi = sysconfig.get_config_var("SOABI").replace(".", "_").replace("-", "_")
-        assert [abi] == list(tags._generic_abi())
-
-    def test__generic_abi(self, monkeypatch):
+    def test__generic_abi_macos(self, monkeypatch):
         monkeypatch.setattr(
-            sysconfig, "get_config_var", lambda key: "cpython-37m-darwin"
+            sysconfig, "get_config_var", lambda key: ".cpython-37m-darwin.so"
         )
-        assert list(tags._generic_abi()) == ["cpython_37m_darwin"]
+        monkeypatch.setattr(tags, "interpreter_name", lambda: "cp")
+        assert tags._generic_abi() == ["cp37m"]
 
-    def test__generic_abi_no_soabi(self, monkeypatch):
-        monkeypatch.setattr(sysconfig, "get_config_var", lambda key: None)
-        assert not list(tags._generic_abi())
+    def test__generic_abi_linux_cpython(self, monkeypatch):
+        config = {
+            "Py_DEBUG": False,
+            "WITH_PYMALLOC": True,
+            "EXT_SUFFIX": ".cpython-37m-x86_64-linux-gnu.so",
+        }
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        monkeypatch.setattr(tags, "interpreter_name", lambda: "cp")
+        # They are identical
+        assert tags._cpython_abis((3, 7)) == ["cp37m"]
+        assert tags._generic_abi() == ["cp37m"]
+
+    def test__generic_abi_jp(self, monkeypatch):
+        config = {"EXT_SUFFIX": ".return_exactly_this.so"}
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        assert tags._generic_abi() == ["return_exactly_this"]
+
+    def test__generic_abi_graal(self, monkeypatch):
+        config = {"EXT_SUFFIX": ".graalpy-38-native-x86_64-darwin.so"}
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        assert tags._generic_abi() == ["graalpy_38_native"]
+
+    def test__generic_abi_none(self, monkeypatch):
+        config = {"EXT_SUFFIX": "..so"}
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        assert tags._generic_abi() == []
+
+    @pytest.mark.parametrize("ext_suffix", ["invalid", None])
+    def test__generic_abi_error(self, ext_suffix, monkeypatch):
+        config = {"EXT_SUFFIX": ext_suffix}
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        with pytest.raises(SystemError) as e:
+            tags._generic_abi()
+        assert "EXT_SUFFIX" in str(e.value)
+
+    def test__generic_abi_linux_pypy(self, monkeypatch):
+        # issue gh-606
+        config = {
+            "Py_DEBUG": False,
+            "EXT_SUFFIX": ".pypy39-pp73-x86_64-linux-gnu.so",
+        }
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        monkeypatch.setattr(tags, "interpreter_name", lambda: "pp")
+        assert tags._generic_abi() == ["pypy39_pp73"]
+
+    def test__generic_abi_old_windows(self, monkeypatch):
+        config = {
+            "EXT_SUFFIX": ".pyd",
+            "Py_DEBUG": 0,
+            "WITH_PYMALLOC": 0,
+        }
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        assert tags._generic_abi() == tags._cpython_abis(sys.version_info[:2])
+
+    def test__generic_abi_windows(self, monkeypatch):
+        config = {
+            "EXT_SUFFIX": ".cp310-win_amd64.pyd",
+        }
+        monkeypatch.setattr(sysconfig, "get_config_var", config.__getitem__)
+        assert tags._generic_abi() == ["cp310"]
+
+    @pytest.mark.skipif(sys.implementation.name != "cpython", reason="CPython-only")
+    def test__generic_abi_agree(self):
+        """Test that the two methods of finding the abi tag agree"""
+        assert tags._generic_abi() == tags._cpython_abis(sys.version_info[:2])
 
     def test_generic_platforms(self):
         platform = sysconfig.get_platform().replace("-", "_")
         platform = platform.replace(".", "_")
         assert list(tags._generic_platforms()) == [platform]
 
+    def test_generic_platforms_space(self, monkeypatch):
+        """Ensure platform tags normalize spaces to underscores."""
+        platform_ = "isilon onefs"
+        monkeypatch.setattr(sysconfig, "get_platform", lambda: platform_)
+        assert list(tags._generic_platforms()) == [platform_.replace(" ", "_")]
+
     def test_iterator_returned(self):
         result_iterator = tags.generic_tags("sillywalk33", ["abi"], ["plat1", "plat2"])
         assert isinstance(result_iterator, collections.abc.Iterator)
@@ -874,7 +945,7 @@ class TestGenericTags:
         assert result == [tags.Tag("sillywalkNN", "none", "any")]
 
     def test_abis_default(self, monkeypatch):
-        monkeypatch.setattr(tags, "_generic_abi", lambda: iter(["abi"]))
+        monkeypatch.setattr(tags, "_generic_abi", lambda: ["abi"])
         result = list(tags.generic_tags(interpreter="sillywalk", platforms=["any"]))
         assert result == [
             tags.Tag("sillywalk", "abi", "any"),
